//**********************************************************/
//* Practica de Procesadores de Lenguaje                   */
//* Curso 2008 - 2009                                      */
//* Diseo e implementacin de un compilador de           */
//* Fichero LEX                                            */
//*                                                        */
//* Autor: David Martinez Pea                             */
//*        davimartinezpenya@hotmail.com                   */
//*        629 047 152                                     */
//**********************************************************/

//Primera parte, Cdigo de usuario, se copia al inicio

package compiler.lexical;

import compiler.syntax.sym;
import compiler.lexical.Token;
import es.uned.compiler.lexical.LexicalError;
import es.uned.compiler.lexical.LexicalErrorManager;

// incluir aqui, si es necesario otras importaciones
//import java_cup.runtime.Symbol;

%%
//Segunda parte, Directivas Jlex, macros, configuracin de comportamiento
 
%public
%class Scanner
%char
%line
%column
%cup

// incluir aqui, si es necesario otras directivas
%unicode
%state COMENTARIO

%{
  private int commentCount = 0;
%}  

%eof{
	if (commentCount>0) {
                            // error en caso de no tener los comentarios bien balanceados
                            LexicalError error = new LexicalError ();
                            error.setLine (yyline + 1);
                            error.setColumn (yycolumn + 1);
                            error.setLexema (yytext ());
                            lexicalErrorManager.lexicalFatalError ("Comentarios mal balanceados, falta cerrar "+ commentCount + " comentario/s ", error);
                        }
%eof}

%{
  LexicalErrorManager lexicalErrorManager = new LexicalErrorManager ();
%}  

LETRA=[a-zA-Z]
DIGITO=[0-9]
CTE_ENTERA={DIGITO}+
ESPACIO=([\ \t\b\r\n\f])+
COMENTARIOIZQ=("/*")
COMENTARIODER=("*/")
CTE_CADENA = [^\"]*

//CTE_CADENA=\"([^\n\b\r\t\f\"])*\"
IDENTIFICADOR = ({LETRA}({LETRA}|{DIGITO}|("_"))*)
NOIDENTIFICADOR = {CTE_ENTERA}{IDENTIFICADOR}

%%
//Tercer parte, reglas patrn-accin

//***************/
//* Comentarios */
//***************/ 

<YYINITIAL> 
{

//******************************/
//* COMENTARIOS                */
//******************************/

  //si en el estado inicial encontramos /*, cambiamos al estado COMENTARIO
  {COMENTARIOIZQ} {yybegin(COMENTARIO); commentCount = commentCount + 1; /*lexicalErrorManager.lexicalDebug ("Abrimos comentarios")*/;}

  //Si encontramos el cierre de un comentario sin su apertura, mostramos error
  {COMENTARIODER} {LexicalError error = new LexicalError (); error.setLine (yyline + 1); error.setColumn (yycolumn + 1); error.setLexema (yytext ()); lexicalErrorManager.lexicalError ("Se ha encontrado un cierre de comentarios [" + (yyline + 1) + ";" + (yycolumn + 1) + "], pero no hay comentarios abiertos.", error);}

//******************************/
//* DELIMITADORES              */
//******************************/

  "("				{  Token token = new Token (sym.PARI); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  ")"				{  Token token = new Token (sym.PARD); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "{"				{  Token token = new Token (sym.LLAI); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "}"				{  Token token = new Token (sym.LLAD); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  ","				{  Token token = new Token (sym.COMA); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  ";"				{  Token token = new Token (sym.PYCO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  ":"				{  Token token = new Token (sym.DOSP); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "&"				{  Token token = new Token (sym.ANDP); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}

//******************************/
//* OPERADORES ARITMETICOS     */
//******************************/

  "+"				{  Token token = new Token (sym.MAS); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "/"				{  Token token = new Token (sym.DIV); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}

//******************************/
//* OPERADORES RELACIONALES    */
//******************************/

  "<"				{  Token token = new Token (sym.MENOR); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "!="				{  Token token = new Token (sym.DISTINTO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "!"				{  Token token = new Token (sym.NEGACION); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}

//******************************/
//* OPERADORES LOGICOS         */
//******************************/

  "&&"				{  Token token = new Token (sym.AND); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}

//******************************/
//* OTROS OPERADORES           */
//******************************/

  "="	  			{  Token token = new Token (sym.ASIGNACION); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "++"	 			{  Token token = new Token (sym.AUTOINCREMENTO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "+="	 			{  Token token = new Token (sym.AUTOSUMA); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}

//******************************/
//* OPERADORES DE ACCESO       */
//******************************/
// Lo dejo comentado, en un principio el tipo B tiene registros, pero no vectores.
//  "[" 			{  Token token = new Token (sym.CORCHI); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
//  "]" 			{  Token token = new Token (sym.CORCHD); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "." 			{  Token token = new Token (sym.PUNTO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}

//******************************/
//* IDENTIFICADORES Y PALABRAS */
//* RESERVADAS                 */
//******************************/

  "caso"            {  Token token = new Token (sym.CASO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "#constante"      {  Token token = new Token (sym.CONSTANTE); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "corte"           {  Token token = new Token (sym.CORTE); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "entero" 			{  Token token = new Token (sym.ENTERO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "enumera"         {  Token token = new Token (sym.ENUMERA); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "escribe"         {  Token token = new Token (sym.ESCRIBE); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "escribeEnt"      {  Token token = new Token (sym.ESCRIBEENT); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "alternativas"    {  Token token = new Token (sym.ALTERNATIVAS); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "para"            {  Token token = new Token (sym.PARA); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "pordefecto"      {  Token token = new Token (sym.PORDEFECTO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "principal" 		{  Token token = new Token (sym.PRINCIPAL); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "registro" 		{  Token token = new Token (sym.REGISTRO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "devuelve" 		{  Token token = new Token (sym.DEVUELVE); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "salto"           {  Token token = new Token (sym.SALTO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "si" 				{  Token token = new Token (sym.SI); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "sino" 			{  Token token = new Token (sym.SINO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "tipo" 			{  Token token = new Token (sym.TIPO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  "vacio" 			{  Token token = new Token (sym.VACIO); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
    
//******************************/
//* LITERALES                  */
//******************************/

  \"{CTE_CADENA}\"  {  Token token = new Token (sym.CTE_CADENA); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  {CTE_ENTERA} 		{  
                       //Comprobamos que el entero est dentro del rango permitido
                       int valorEntero;
                       double valorDouble = (new Double(yytext())).doubleValue();
                       if (valorDouble > Integer.MAX_VALUE)
                            {
                            LexicalError error = new LexicalError ();
                            error.setLine (yyline + 1);
                            error.setColumn (yycolumn + 1);
                            error.setLexema (yytext ());
                            lexicalErrorManager.lexicalError ("El entero introducido supera la capacidad mxima de los nmeros enteros, se tomar el valor " + Integer.toString(Integer.MAX_VALUE) + " para evitar errores de desbordamiento", error);
                            valorEntero = Integer.MAX_VALUE;
                            }
                        else valorEntero = (new Integer(yytext())).intValue();
                        Token token = new Token (sym.CTE_ENTERA); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (Integer.toString(valorEntero)); return token;
                    }
  {IDENTIFICADOR}   {  Token token = new Token (sym.IDENTIFICADOR); token.setLine (yyline + 1); token.setColumn (yycolumn + 1); token.setLexema (yytext ()); return token;}
  {NOIDENTIFICADOR} {
                       LexicalError error = new LexicalError ();
                       error.setLine (yyline + 1);
                       error.setColumn (yycolumn + 1);
                       error.setLexema (yytext ());
                       lexicalErrorManager.lexicalError ("Todos los identificadores en  deben comenzar por una letra del alfabeto  [" + (yyline + 1) + ";" + (yycolumn + 1) + "]", error);
                    }


    // incluir aqui el resto de las reglas patron - accion
    
//******************************/
//* Ignorar espacios en blanco */
//******************************/
  {ESPACIO}         {
                    }

// error en caso de coincidir con ningn patrn
	[^]     
                    {
                       LexicalError error = new LexicalError ();
                       error.setLine (yyline + 1);
                       error.setColumn (yycolumn + 1);
                       error.setLexema (yytext ());
                       lexicalErrorManager.lexicalError (yytext() + ": Token no vlido en el fichero origen [" + (yyline + 1) + ":" + (yycolumn + 1) +"]" , error);
                    }
}

//En el estado COMENTARIO todo excepto los delimitadores /* y */) es omitido
<COMENTARIO>
{
    //En el estado COMENTARIO si aparece de nuevo un /*, incrementamos el contador
    {COMENTARIOIZQ} {commentCount++;}

    //Si encontramos */ decrementamos el contador y comprobamos si ha terminado el comentario
    {COMENTARIODER} {commentCount--; if (commentCount == 0) {/*lexicalErrorManager.lexicalDebug ("Cerramos comentarios")*/; yybegin(YYINITIAL);}}
				
    //Cualquier otra cosa es omitida
    [^]             {}
}