
package compiler;

import es.uned.compiler.code.FinalCodeFactory;
import es.uned.compiler.code.FinalCodeFactoryIF;
import es.uned.compiler.semantic.SemanticErrorManager;
import es.uned.compiler.semantic.symbol.ScopeManager;
import es.uned.compiler.semantic.symbol.ScopeManagerIF;
import es.uned.compiler.syntax.SyntaxErrorManager;

/**
 * Clase que implementa un "contexto" para la prctica. 
 * Permite acceder en todo momento a los gestores (manager) definidos en la
 * arquitectura.
 * No han de alterarse los mtodos incluidos.
 *
 */
public class CompilerContext
{
    private static SyntaxErrorManager   syntaxErrorManager   = new SyntaxErrorManager ();
    private static SemanticErrorManager semanticErrorManager = new SemanticErrorManager ();
    private static ScopeManagerIF       scopeManager         = new ScopeManager ();
    private static FinalCodeFactoryIF   finalCodeFactory     = new FinalCodeFactory ();
    
    /**
     * Returns the scopeManager.
     * @return Returns the scopeManager.
     */
    public static ScopeManagerIF getScopeManager ()
    {
        return scopeManager;
    }
    
    /**
     * Returns the semanticErrorManager.
     * @return Returns the semanticErrorManager.
     */
    public static SemanticErrorManager getSemanticErrorManager ()
    {
        return semanticErrorManager;
    }

    /**
     * Returns the syntaxErrorManager.
     * @return Returns the syntaxErrorManager.
     */
    public static SyntaxErrorManager getSyntaxErrorManager ()
    {
        return syntaxErrorManager;
    }

    /**
     * Returns the finalCodeFactory.
     * @return Returns the finalCodeFactory.
     */
    public static FinalCodeFactoryIF getFinalCodeFactory ()
    {
        return finalCodeFactory;
    }
}
