package compiler.code;

import compiler.CompilerContext;
import es.uned.compiler.code.ExecutionEnvironmentIF;
import es.uned.compiler.intermediate.CuadrupleIF;
import es.uned.compiler.intermediate.OperandIF;
import compiler.intermediate.Value;
import compiler.intermediate.Temporal;
import compiler.intermediate.Variable;
import compiler.intermediate.Registro;
import compiler.semantic.symbol.SymbolParameter;
import compiler.semantic.symbol.SymbolVariable;
import es.uned.compiler.semantic.symbol.ScopeManagerIF;
import es.uned.compiler.semantic.symbol.SymbolIF;
import java.util.ArrayList;
import java.util.List;

/**
 * Clase que representa el entorno de ejecucin del ensamblador ENS2001
 * 
 * El estudiante ha de implementar el mtodo translate
 *
 */
public class ENS2001Environment
        implements ExecutionEnvironmentIF {

    List cadenasTxt = new ArrayList();
	ScopeManagerIF            scopeManager         = CompilerContext.getScopeManager ();

    /**
     * Constructor for ENS2001Environment.
     */
    public ENS2001Environment() {
        cadenasTxt.add("\"\\n\"");
    }

    /**
     * Translate the intermediate code operation into a final code operation. 
     * @param operation The intermediate code operation to translate.
     * @return The translation of the intermediate code operation.
     */
    public String translate(CuadrupleIF cuadruple) {
        String trad = null;
        String op = cuadruple.getOperation();
        if (op.equals("ADD")) {
            trad = tradADD(cuadruple);
        } else if (op.equals("ASIG")) {
            trad = tradASIG(cuadruple);
        } else if (op.equals("BN")) {
            trad = tradBN(cuadruple);
        } else if (op.equals("BR")) {
            trad = tradBR(cuadruple);
        } else if (op.equals("BZ")) {
            trad = tradBZ(cuadruple);
        } else if (op.equals("BNZ")) {
            trad = tradBNZ(cuadruple);
        } else if (op.equals("CMP")) {
            trad = tradCMP(cuadruple);
        } else if (op.equals("DEV")) {
            trad = tradDEV(cuadruple);
        } else if (op.equals("DIV")) {
            trad = tradDIV(cuadruple);
        } else if (op.equals("ESCRIBE")) {
            trad = tradESCRIBE(cuadruple);
        } else if (op.equals("ESCRIBEENT")) {
            trad = tradESCRIBEENT(cuadruple);
        } else if (op.equals("ESCRIBESAL")) {
            trad = tradESCRIBESAL(cuadruple);
        } else if (op.equals("ETIQUETA")) {
            trad = tradETIQUETA(cuadruple);
        } else if (op.equals("FIN")) {
            trad = tradFIN(cuadruple);
            trad = trad + tradCadenasTxt();
        } else if (op.equals("FINFUNCION")) {
            trad = ";#### fin funcion";
        } else if (op.equals("FUNCION")) {
            trad = ";#### def funcion";
        } else if (op.equals("INICIO")) {
            trad = tradINICIO(cuadruple);
        } else if (op.equals("LLAMA")) {
            trad = tradLLAMA(cuadruple);
        } else if (op.equals("NOP")) {
            trad = tradNOP(cuadruple);
        } else if (op.equals("PAR")) {
            trad = tradPAR(cuadruple);
        } else if (op.equals("PARAMETROS")) {
            trad = tradPARAMETROS(cuadruple);
        } else {
            trad = "FUNCIN DE TRADUCCIN NO IMPLEMENTADA TODAVIA";
        }
        //IMPORTANTE: a implementar por el estudiante
        //return "\t\t;" + cuadruple.toString() +
        return trad;
    }

    private String tradADD(CuadrupleIF cuadruple) {
        String op1 = obtenOperando(cuadruple.getFirstOperand());
        String op2 = obtenOperando(cuadruple.getSecondOperand());
        String res = obtenOperando(cuadruple.getResult());
        String trad = "; Suma " + op1 + " a " + op2 + " y lo guarda en " + res;
        trad = trad + "\n\t" + "ADD " + op1 + ", " + op2;
        trad = trad + "\n\t" + "MOVE " + ".A, " + res;
        return trad;
    }

    private String tradASIG(CuadrupleIF cuadruple) {
        String trad = "";
        String op1 = obtenOperando(cuadruple.getFirstOperand());
        String op2 = obtenOperando(cuadruple.getResult());
        trad = "; Asignamos " + op1 + " a " + op2;
        trad = trad + "\n\tMOVE " + op1 + ", " + op2;
        return trad;
    }

    private String tradBN(CuadrupleIF cuadruple) {
        String trad = "";
        String res = eliminaPuntos(cuadruple.getResult().toString());
        trad = "; Saltamos si es negativo a la etiqueta " + res;
        trad = trad + "\n\tBN $" + res;
        return trad;
    }

    private String tradBNZ(CuadrupleIF cuadruple) {
        String trad = null;
        String res = eliminaPuntos(cuadruple.getResult().toString());
        trad = "; Saltamos si no es negativo a la etiqueta " + res;
        trad = trad + "\n\tBNZ $" + res;
        return trad;
    }

    private String tradBR(CuadrupleIF cuadruple) {
        String trad = null;
        String res = eliminaPuntos(cuadruple.getResult().toString());
        trad = "; Salta incondicional a la etiqueta " + res;
        trad = trad + "\n\tBR $" + res;
        return trad;
    }

    private String tradBZ(CuadrupleIF cuadruple) {
        String trad = null;
        String res = eliminaPuntos(cuadruple.getResult().toString());
        trad = "; Salta si es cero a la etiqueta " + res;
        trad = trad + "\n\tBZ $" + res;
        return trad;
    }

    private String tradCMP(CuadrupleIF cuadruple) {
        String trad = null;
        String op1 = obtenOperando(cuadruple.getFirstOperand());
        String op2 = obtenOperando(cuadruple.getResult());
        trad = "; Comparamos " + op1 + " a " + op2;
        trad = trad + "\n\tCMP " + op1 + ", " + op2;
        return trad;
    }

    private String tradDEV(CuadrupleIF cuadruple) {
        String trad = "";
//        String op1 = obtenOperando(cuadruple.getFirstOperand());
//        String op2 = obtenOperando(cuadruple.getResult());
//        trad = "; Comparamos " + op1 + " a " + op2;
//        trad = trad + "\n\tCMP " + op1 + ", " + op2;
        return trad;
    }

    private String tradDIV(CuadrupleIF cuadruple) {
        String op1 = obtenOperando(cuadruple.getFirstOperand());
        String op2 = obtenOperando(cuadruple.getSecondOperand());
        String res = obtenOperando(cuadruple.getResult());
        String trad = "; Divide " + op1 + " entre " + op2 + " y lo guarda en " + res;
        trad = trad + "\n\t" + "DIV " + op1 + ", " + op2;
        trad = trad + "\n\t" + "MOVE " + ".A, " + res;
        return trad;
    }

    private String tradESCRIBE(CuadrupleIF cuadruple) {
        String trad = null;
        String res = cuadruple.getResult().toString();
        trad = "; Escribimos en pantalla la cadena txt" + (cadenasTxt.size());
        trad = trad + "\n\t" + "WRSTR /txt" + cadenasTxt.size();
        cadenasTxt.add(res);
        return trad;
    }

    private String tradESCRIBEENT(CuadrupleIF cuadruple) {
        String trad = null;
        String res = obtenOperando(cuadruple.getResult());
        trad = "; Escribimos en pantalla " + res;
        trad = trad + "\n\t" + "WRINT " + res;
        return trad;
    }

    private String tradESCRIBESAL(CuadrupleIF cuadruple) {
        String trad = null;
        trad = "; Escribimos un salto de linea ";
        trad = trad + "\n\t" + "WRSTR /txt0";
        return trad;
    }

    private String tradETIQUETA(CuadrupleIF cuadruple) {
        String etiq = eliminaPuntos(cuadruple.getResult().toString());
        String trad = "; Definimos la posicin de la etiqueta " + etiq;
        trad = trad + "\n\t" + etiq + " :";
        return trad;
    }

    private String tradFIN(CuadrupleIF cuadruple) {
        String trad = "; Fin del Cdigo ensamblado de  para ens2001";
        trad = trad + "\n\t" + "HALT";
        return trad;
    }

    private String tradINICIO(CuadrupleIF cuadruple) {
        Integer tamIni = new Integer(cuadruple.getResult().toString());
        String trad = "; Cdigo ensamblado de  para ens2001";
        trad = trad + "\n\t" + "MOVE #65535, .SP";
        trad = trad + "\n\t" + "MOVE .SP, .IY";
        trad = trad + "\n\t" + "SUB .SP, #" + (tamIni + 1) + ""; //TODO: este ms 1 es por el valor devuelto?
        trad = trad + "\n\t" + "MOVE .A, .SP";
        return trad;
    }

    private String tradLLAMA(CuadrupleIF cuadruple) {
        String trad = "";
//        String op1 = obtenOperando(cuadruple.getFirstOperand());
//        String op2 = obtenOperando(cuadruple.getResult());
//        trad = "; Comparamos " + op1 + " a " + op2;
//        trad = trad + "\n\tCMP " + op1 + ", " + op2;
        return trad;
    }

    private String tradNOP(CuadrupleIF cuadruple) {
        String trad = "; No hace nada";
        trad = trad + "\n\t" + "NOP";
        return trad;
    }

    private String tradPAR(CuadrupleIF cuadruple) {
        String trad = "";
//        String op1 = obtenOperando(cuadruple.getFirstOperand());
//        String op2 = obtenOperando(cuadruple.getResult());
//        trad = "; Comparamos " + op1 + " a " + op2;
//        trad = trad + "\n\tCMP " + op1 + ", " + op2;
        return trad;
    }

    private String tradPARAMETROS(CuadrupleIF cuadruple) {
        String trad = "";
        String op1 = obtenOperando(cuadruple.getFirstOperand());
        trad = "; Aadiremos " + op1 + " parmetros";
        //trad = trad + "\n\tPUSH " + op1 + ", " + op2;
        return trad;
    }
    private String obtenOperando(OperandIF op) {
        String trad = "";
        if (op instanceof Value) {
            Value cte = (Value) op;
            trad = "#" + cte.getValue();
        } else if (op instanceof Variable) {
            Variable var = (Variable) op;
            //original, solo variables.
            //SymbolVariable SimVar = (SymbolVariable) var.getScope().getSymbolTable().getSymbol(var.getName());
            //despus para variables y registros
            SymbolIF simBas = var.getScope().getSymbolTable().getSymbol(var.getName());
            //TODO: OJO! las globales como las encuentro?
            //SymbolIF simBas = (SymbolIF) (scopeManager.searchSymbol(var.getName()));
            if (simBas instanceof SymbolVariable){
                SymbolVariable sim = (SymbolVariable) simBas;
                trad = "#-" + sim.getDireccion() + "[.IY]";
            } else {
                SymbolParameter sim = (SymbolParameter) simBas;
                trad = "#+" + sim.getDireccion() + "[.IY]";
            }
        } else if (op instanceof Registro) {
            Registro var = (Registro) op;
            //TODO: OJO! las globales como las encuentro?
            SymbolVariable simVar = (SymbolVariable) var.getScope().getSymbolTable().getSymbol(var.getName());
            trad = "#-" + (simVar.getDireccion() + var.getOffset()) + "[.IY]";
        } else {
            Temporal temp = (Temporal) op;
            trad = "#-" + temp.getDesplazamiento() + "[.IY]";
        }
        return trad;
    }

    private String eliminaPuntos(String cadena) {
        return cadena.replace(".", "");
    }

    private String tradCadenasTxt() {
        String trad = "\n; Cadenas empleadas para mostrar por consola";
        for (int i = 0; i < cadenasTxt.size(); i++) {
            trad = trad + "\n\t" + "txt" + i + ":\t\tDATA " + cadenasTxt.get(i).toString();
        }
        return trad;
    }
}
