package compiler.intermediate;

import es.uned.compiler.intermediate.OperandIF;
import es.uned.compiler.semantic.symbol.ScopeIF;

/**
 * Clase que representa una variable en el cdigo intermedio
 *
 */
public class Variable
    implements OperandIF 
{
    String  name  = null;
    ScopeIF scope = null;
    
    /**
     * Constructor for Variable.
     * @param name The name.
     */
    public Variable (String name)
    {
        this.name = name;
    }
    
    /**
     * Constructor for Variable.
     * @param name The name.
     * @param scope The scope index.
     */
    public Variable (String name, ScopeIF scope)
    {
        this (name);
        this.scope = scope;
    }

    /**
     * Returns the name.
     * @return Returns the name.
     */
    public String getName ()
    {
        return name;
    }

    /**
     * Sets The name.
     * @param name The name to set.
     */
    public void setName (String name)
    {
        this.name = name;
    }

    /**
     * Returns the scope.
     * @return Returns the scope.
     */
    public ScopeIF getScope ()
    {
        return scope;
    }

    /**
     * Sets The scope.
     * @param scope The scope to set.
     */
    public void setScope (ScopeIF scope)
    {
        this.scope = scope;
    }
    
    /**
     * Compares this object with another one.
     * @param other the other object.
     * @return true if two objects has the same properties.
     */
    public boolean equals (Object other)
    {
        if (this == other)
        {
            return true;
        }
        if (!(other instanceof Variable))
        {
            return false;
        }
        
        final Variable anOperandSymbol = (Variable) other;
        return (scope == anOperandSymbol.scope) &&
               ((name == anOperandSymbol.name) ||
                (name.equals (anOperandSymbol.name)));
    }

    /**
     * Returns a hash code for the object.
     */
    public int hashCode ()
    {
        return 255 * scope.hashCode() +
               ((name != null)? name.hashCode () : 0);
    } 

    /**
     * Return a string representing the object.
     * @return a string representing the object.
     */
    public String toString ()
    {        
        return name;
    }
}
