package compiler.semantic.symbol;

import java.util.List;
import es.uned.compiler.semantic.symbol.ScopeIF;
import es.uned.compiler.semantic.symbol.SymbolIF;
import es.uned.compiler.semantic.symbol.SymbolTableIF;
import java.util.HashMap;

/**
 * Clase que representa una tabla de simbolos.
 * 
 * Ha de ser implementada por el estudiante
 *
 */
public class SymbolTable
        implements SymbolTableIF {

    private ScopeIF scope = null;
    private HashMap tablaDeSimbolos = new HashMap();

    public HashMap getTablaDeSimbolos() {
        return tablaDeSimbolos;
    }

    public void setTablaDeSimbolos(HashMap tablaDeSimbolos) {
        this.tablaDeSimbolos = tablaDeSimbolos;
    }

    /**
     * Constructor for TypeTable.
     */
    public SymbolTable() {
    }

    /**
     * Constructor for TypeTable.
     * @param The scope of the symbol table.
     */
    public SymbolTable(ScopeIF scope) {
        this.scope = scope;
    }

    /**
     * Returns the scope of this Symbol table. 
     * @return the scope of this Symbol table. 
     */
    public ScopeIF getScope() {
        return scope;
    }

    /**
     * Returns a symbol from the symbol table.
     * @param name the symbol key.
     */
    public SymbolIF getSymbol(String name) {
        Object simbolo = this.getTablaDeSimbolos().get(name);
        if (simbolo != null && simbolo instanceof SymbolIF) {
            return (SymbolIF) simbolo;
        } else {
            return null;
        }
    }

    /**
     * Sets a symbol to the symbol table.
     * @param name the symbol name.
     * @param symbol the symbol.
     */
    public void setSymbol(String name, SymbolIF symbol) {
        this.getTablaDeSimbolos().put(name, symbol);
    }

    /**
     * Returns the list of symbols.
     * @return A list of symbols.
     */
    public List getSymbols() {
        return compiler.semantic.util.Utilidades.getListFromHashMap (this.getTablaDeSimbolos());
    }

    /**
     * Indicates whether the symbol is contained in the symbol table.
     * @param symbol The symbol. 
     * @return True if the symbol is contained in the symbol table.
     */
    public boolean containsSymbol(SymbolIF symbol) {
        return this.getTablaDeSimbolos().containsValue(symbol);
    }

    /**
     * Indicates whether the symbol is contained in the symbol table.
     * @param name The symbol name. 
     * @return True if the symbol is contained in the symbol table.
     */
    public boolean containsSymbol(String name) {
        return this.getTablaDeSimbolos().containsKey(name);
    }

    /**
     * Compares this object with another one.
     * @param other the other object.
     * @return true if two objects has the same properties.
     */
    public boolean equals(Object other) {
        // IMPORTANTE: a implementar por el estudiante
        return super.equals(other);
    }

    /**
     * Returns a hash code for the object.
     */
    public int hashCode() {
        // IMPORTANTE: a implementar por el estudiante
        return super.hashCode();
    }

    /**
     * Return a string representing the object.
     * @return a string representing the object.
     */
    public String toString() {
        // IMPORTANTE: a implementar por el estudiante
        return super.toString();
    }
}
