package compiler.semantic.type;

import compiler.CompilerContext;
import compiler.semantic.symbol.SymbolConstant;
import es.uned.compiler.semantic.symbol.ScopeIF;
import es.uned.compiler.semantic.type.TypeBase;
import es.uned.compiler.semantic.type.TypeIF;
import compiler.syntax.nonTerminal.ListaObjetos;
import compiler.syntax.nonTerminal.Var;
import es.uned.compiler.semantic.SemanticErrorManager;
import es.uned.compiler.semantic.symbol.SymbolTableIF;
import es.uned.compiler.semantic.type.TypeTableIF;
import java.util.Iterator;

/**
 * Clase que representa un tipo enumerado
 * Ha de ser implementada por el estudiante
 *
 */
public class TypeEnum
        extends TypeBase {

    private ListaObjetos identificadores = null;
    SemanticErrorManager semanticErrorManager = CompilerContext.getSemanticErrorManager();

    /**
     * Constructor for TypeEnum.
     * @param scope The scope.
     */
    public TypeEnum(ScopeIF scope) {
        super(scope);
    }

    /**
     * Constructor for TypeEnum.
     * @param scope The scope
     * @param name The name.
     */
    public TypeEnum(ScopeIF scope, String name) {
        super(scope, name);
    }

    public TypeEnum(ScopeIF scope, String name, ListaObjetos ides) {
        super(scope, name);
        identificadores = new ListaObjetos();
        int i;
        int j = 0;

        for (i = ides.size() - 1; i >= 0; i--) {
            //en ides solo hay nombres, aqu les asignamos valor en orden ascendente desde el 0
            //Var v = new Var((String) ides.get(i), j, 0, 0);
            Var v = (Var) ides.get(i);
            v.setValue(j);
            j++;
            if (addEnum(v)) {
                SymbolTableIF symbolTable = scope.getSymbolTable();
                TypeTableIF typeTable = scope.getTypeTable();
                SymbolConstant nuevosimbolo = new SymbolConstant(scope, v.getName(), typeTable.getType("entero"), v.getValue());
                if (!symbolTable.containsSymbol(nuevosimbolo)) {
                    symbolTable.setSymbol(v.getName(), nuevosimbolo);
                } else {
                    semanticErrorManager.semanticFatalError("El elemento " + v.getName() + " [" + v.getLin() + ":" + v.getCol() + "], ya esta definido.");
                }
            } else {
                semanticErrorManager.semanticFatalError("El elemento " + v.getName() + " [" + v.getLin() + ":" + v.getCol() + "], ya esta definido dentro del tipo enumerado " + name + ".");
            }
        }
    }
    public Integer getSize(){
        //####MEMORIA: No se si necesitare el tamao del tipo funcion.
        return 1;
    }
    //Aade un nuevo identificador
    public boolean addEnum(Var id) {
        if (validaEnum(id)) {
            return identificadores.add(id);
        } else {
            return false;
        }
    }

    //Comprueba si existen un identificador. Devuelve true si no est en el tipo enumerado.
    public boolean validaEnum(Var id) {
        boolean valido = true;
        Var elemento;
        if (identificadores == null || identificadores.size() == 0) {
            return valido;
        } else {
            int i = 0;
            while (valido && i < identificadores.size()) {
                elemento = (Var) identificadores.get(i);
                //              semanticErrorManager.semanticInfo ("comparando: " + elemento.getName() + " y " + id.getName());
                if (elemento.getName().equals(id.getName())) {
                    valido = false;
//                    semanticErrorManager.semanticInfo ("son iguales");
                }
//                else {                    semanticErrorManager.semanticInfo ("son diferentes");}
                i++;
            }
            return valido;
        }
    }

    public ListaObjetos getIdentificadores() {
        return identificadores;
    }

    /**
     * Compares this object with another one.
     * @param other the other object.
     * @return true if two objects has the same properties.
     */
    @Override
    public final boolean equals(Object other) {
        // IMPORTANTE: a implementar por el estudiante
        //return super.equals(other);
        if (this == other) {
            return true;
        }
        if (!(other instanceof TypeIF)) {
            return false;
        }
        //A la hora de hacer equals en el tipo enum... ignorar el case?
        return ((TypeIF) other).getName().equalsIgnoreCase(this.getName());
    }

    /**
     * Returns a hash code for the object.
     */
    @Override
    public final int hashCode() {
        // IMPORTANTE: a implementar por el estudiante
        //return super.hashCode();
        return 255 * getName().hashCode() + getScope().hashCode();
    }

    /**
     * Return a string representing the object.
     * @return a string representing the object.
     */
    @Override
    public String toString() {
        // IMPORTANTE: a implementar por el estudiante
        //return super.toString();
        String str= "TypeEnum del ambito " + getScope().getName() + " con el nombre: " + this.getName() + " y los identificadores: ";
        int i=0;
        for (i=0; i< this.getIdentificadores().size(); i++) {
            str += "\n      " + ((Var) this.getIdentificadores().get(i)).getName() + " que vale: " + ((Var) this.getIdentificadores().get(i)).getValue();
        }
        return str;
    }
}
