package compiler.semantic.type;

import java.util.List;
import es.uned.compiler.semantic.symbol.ScopeIF;
import es.uned.compiler.semantic.type.TypeIF;
import es.uned.compiler.semantic.type.TypeTableIF;
import java.util.HashMap;

/**
 * Clase que representa la tabla de tipos. 
 * 
 * Ha de ser implementada por el estudiante
 *
 */
public class TypeTable 
    implements TypeTableIF
{

    private ScopeIF scope = null;
    private HashMap tablaDeTipos=new HashMap();

    public HashMap getTablaDeTipos() {
        return tablaDeTipos;
    }

    public void setTablaDeTipos(HashMap tablaDeTipos) {
        this.tablaDeTipos = tablaDeTipos;
    }

    /**
     * Constructor for TypeTable.
     */
    public TypeTable ()
    {       
    }
    
    /**
     * Constructor for TypeTable.
     * @param scope The scope where type table is defined.
     */
    public TypeTable (ScopeIF scope)
    {       
        this.scope = scope;
    }
    
    /**
     * Returns the scope of this Symbol table. 
     * @return the scope of this Symbol table. 
     */
    public ScopeIF getScope ()
    {
        return  scope;
    }
    
    /**
     * Looks up a type by name.
     * @param name The name of the type.
     * @return A type.
     */
    public TypeIF getType(String name) {
        Object tipo = this.getTablaDeTipos().get(name);
        if (tipo != null && tipo instanceof TypeIF) {
            return (TypeIF) tipo;
        } else {
            return null;
        }
    }

    /**
     * Sets a type in the type table.
     * @param type the type.
     */
    public void setType (TypeIF type)
    {
        this.getTablaDeTipos().put(type.getName(),type);
    }
    
    /**
     * Sets a type in the type table.
     * @param name the type name.
     * @param type the type.
     */
    public void setType (String name, TypeIF type)
    {
        this.getTablaDeTipos().put(name, type);
    }
    
    /**
     * Returns the list of types.
     * @return A list of types.
     */
    public List getTypes ()
    {    
        return compiler.semantic.util.Utilidades.getListFromHashMap(this.getTablaDeTipos());
    }
    
    /**
     * Indicates whether the type is contained in the type table.
     * @param type The type. 
     * @return True if the symbol is contained in the type table.
     */
    public boolean containsType (TypeIF symbol)
    {
        return this.getTablaDeTipos().containsKey(symbol.getName());
    }
    
    /**
     * Indicates whether the type is contained in the type table.
     * @param name The type name. 
     * @return True if the type is contained in the type table.
     */
    public boolean containsType (String name)
    {
        return this.getTablaDeTipos().containsKey(name);
    }
    
    /**
     * Compares this object with another one.
     * @param other the other object.
     * @return true if two objects has the same properties.
     */
    public boolean equals (Object other)
    {
      // IMPORTANTE: a implementar por el estudiante
      return super.equals(other);
    }

    /**
     * Returns a hash code for the object.
     */
    public int hashCode ()
    {
       // IMPORTANTE: a implementar por el estudiante
       return super.hashCode();
    } 

    /**
     * Return a string representing the object.
     * @return a string representing the object.
     */
    public String toString ()
    {
       // IMPORTANTE: a implementar por el estudiante
       return super.toString() + "," + this.getTypes().toString();
    } 
}
