
package compiler.test;

import java.io.File;
import java.io.FileReader;
import java.util.Iterator;
import java.util.List;

import compiler.lexical.Scanner;
import compiler.syntax.parser;

import compiler.CompilerContext;

import es.uned.compiler.code.FinalCodeFactoryIF;
import es.uned.compiler.semantic.SemanticErrorManager;
import es.uned.compiler.semantic.symbol.Scope;
import es.uned.compiler.semantic.symbol.ScopeIF;
import es.uned.compiler.semantic.symbol.ScopeManagerIF;
import es.uned.compiler.semantic.symbol.SymbolIF;
import es.uned.compiler.semantic.symbol.SymbolTableIF;
import es.uned.compiler.semantic.type.TypeIF;
import es.uned.compiler.semantic.type.TypeTableIF;
import es.uned.compiler.syntax.SyntaxErrorManager;

/**
 * Esta clase contiene el mtodo main para probar el analizador semntico.
 * Puede modificarse si el alumno lo cree necesario, pero debe mantenerse el
 * nombre de la clase y su formato de invocacin (un parmetro de entrada a main
 * que contenga la ruta de un archivo fuente.
 * Antes de ejecutarla hay que haber llamado a la tarea 'build' de ant.
 */

public class FinalTestCase
{
    private static SemanticErrorManager semanticErrorManager = null;
    private static String sourceFileExtension = ".n";
    
    
    /**
     * Constructor for FinalTestCase.
     */
    public FinalTestCase ()
    {
    }

    /**
     * Tests the parser agains a test file.
     * @param fileName The path of the file test.
     */
    public void testSemantic (String fileName)
    {
        SyntaxErrorManager syntaxErrorManager = new SyntaxErrorManager ();
        
        try 
        {
        	syntaxErrorManager.syntaxInfo ("Fichero de entrada > " + fileName);
        	
        	// Configuracion del codigo final
	    	 FinalCodeFactoryIF finalCodeFactory = CompilerContext.getFinalCodeFactory();
	         String ensName = fileName.replaceFirst(sourceFileExtension,".ens");
	         syntaxErrorManager.syntaxInfo("Fichero a generar > "+ensName);
	         File outputFile = new File (ensName);
	         finalCodeFactory.setFile(outputFile);
            
            
            FileReader aFileReader = new FileReader (fileName);
            Scanner aScanner = new Scanner (aFileReader);
            parser aParser = new parser (aScanner);
            aParser.parse ();
            logSemantics ();
        }
        catch (Exception e)
        {
            e.printStackTrace ();
        }
    }

    /**
     * Logs the semantic info.
     */
    private void logSemantics ()
    {
        logTypeTable ();
        logSymbolTables ();
    }
    
    /**
     * Logs the symbol tables.
     */
    private void logTypeTable ()
    {
    	ScopeManagerIF scopeManager   = CompilerContext.getScopeManager();
        semanticErrorManager.semanticDebug ("********************");
        semanticErrorManager.semanticDebug ("** TABLA DE TIPOS **");
        semanticErrorManager.semanticDebug ("********************");
        // get all scopes
        List scopes = scopeManager.getAllScopes();
        // get type table of each scope
        Iterator it = scopes.iterator();
        Scope scope;
        TypeTableIF tt;
        while (it.hasNext()){
        	scope = (Scope) it.next();
        	logTypeTable(scope.getTypeTable());
        }
    }
    
    /**
     * Log a typetable
     * @param typeTable
     */
    private void logTypeTable(TypeTableIF typeTable){
    	
        List types = typeTable.getTypes ();
        if (types!=null){
	        Iterator typeTableIt = types.iterator ();
	        while (typeTableIt.hasNext ())
	        {
	            TypeIF aType = (TypeIF) typeTableIt.next ();
	            semanticErrorManager.semanticDebug (aType);
	        }
        }
        else{
        	semanticErrorManager.semanticDebug("La tabla de tipos no est implementada correctamente an");
        }
    }
    
    /**
     * Logs all symbol tables.
     */
    private void logSymbolTables ()
    {
        ScopeManagerIF scopeManager   = CompilerContext.getScopeManager();
        semanticErrorManager.semanticDebug("************************");
        semanticErrorManager.semanticDebug("** TABLAS DE SIMBOLOS **");
        semanticErrorManager.semanticDebug("************************");
        
        // get all scopes
        List scopes = scopeManager.getAllScopes();
        // get symbol table of each scope
        Iterator it = scopes.iterator();
        ScopeIF scope;
        SymbolTableIF st;
        while (it.hasNext()){
        	scope = (ScopeIF) it.next();
        	logSymbolTable(scope.getSymbolTable());
	     }
    }

    /**
     * Logs a symbol table.
     * @param symbolTable the symbol table to log.
     */
    private void logSymbolTable (SymbolTableIF symbolTable)
    {     
        semanticErrorManager.semanticDebug("************************");
        semanticErrorManager.semanticDebug("** TABLA DE SIMBOLOS [" + symbolTable.getScope ().getName () + "]");
        semanticErrorManager.semanticDebug("************************");
        List symbols = symbolTable.getSymbols ();
        if (symbols!=null){
	        Iterator symbolTableIt = symbols.iterator ();
	        
	        while (symbolTableIt.hasNext ())
	        {
	            SymbolIF aSymbol = (SymbolIF) symbolTableIt.next ();
	            semanticErrorManager.semanticDebug (aSymbol);
	        }
        }
    }

    /**
     * Starts the parser test case.
     * @param args An 1 element array containing the path to the input file.
     */
    public static void main (String args[])
    {
        if (args.length < 1 || args.length > 1)
        {
            System.err.println ("Uso:  java FinalTestCase ./test/ejemplo."+sourceFileExtension);
        }
        else
        {
            FinalTestCase finalTest = new FinalTestCase ();
            String fileName = args[0];
            semanticErrorManager = CompilerContext.getSemanticErrorManager();
            finalTest.testSemantic (fileName);
        }
    }
}
